/******************************************************************************
    CallWave VoiceMail Widget

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

 ******************************************************************************/

/******************************************************************************
  VM_WidgetViewController class
 ******************************************************************************/

/******************************************************************************
  VM_WidgetViewController C'tor

 ******************************************************************************/
function VM_WidgetViewController( initialShowActivationPreference, initialMaxCalls )
{
    this.m_state            = this.VIEW_STATE_SHOW_REGISTER;

    // Non-persistent account attributes
    this.m_name             = null;
    this.m_emailAddress     = null;
    this.m_phoneNumber      = null;

    // create call list view
    var self = this;
    this.m_callListView     = new VM_CallListView( initialMaxCalls, initialShowActivationPreference, function() { self.OnLoginRequired(arguments[0] ? arguments[0] : '' ) } );

    // create the register view
    var onRegisterAccountSuccess = function( responseMessage )
    {
        self.setRegistrationSucceeded( );
    };

    this.m_registerView = new VM_RegisterView( onRegisterAccountSuccess );

    // create login view
    var onAuthenticateSuccess = function( )
    {
        self.setLoginSucceeded( );
    };

    var onGetAccountInfoSucceeded = function( responseMessage )
    {
        self.OnAuthenticationSucceeded( responseMessage );
    };

    this.m_loginView = new VM_LoginView( onAuthenticateSuccess, onGetAccountInfoSucceeded, function() { self.OnLoginRequired( arguments[0] ? arguments[0] : '') } );

    // perform initialization
    var uString = VM_GetPreference( PREF_KEY_U_STRING );
    var account = VM_GetPreference( PREF_KEY_ACCOUNT );

    if (  account == null || account.length == 0 ||
          uString == null || uString.length == 0 )
    {
        this.setRegistrationRequired( );
    }
    else
    {
        this.setAutoLogin( );
    }
}
VM_WidgetViewController.prototype =
{
    /**************************************************************************
      View State enum
     **************************************************************************/

    VIEW_STATE_SHOW_REGISTER   : 0,
    VIEW_STATE_SHOW_LOGIN      : 1,
    VIEW_STATE_SHOW_CALL_LIST  : 2,
    VIEW_STATE_DEFAULT         : 0,

    /**************************************************************************
      Accessors
     **************************************************************************/
    resetAccountInfo:function( )
    {
        this.m_name             = null;
        this.m_emailAddress     = null;
        this.m_phoneNumber      = null;

        this.m_callListView.ResetAccountInfo( );

        VM_SetPreference( PREF_KEY_ACCOUNT,  '' );
        VM_SetPreference( PREF_KEY_U_STRING, '' );
    },

    getCallListView:function( )
    {
        return this.m_callListView;
    },

    UpdatePreferences : function( maxCalls, showActivation )
    {
        try
        {
            VM_SetPreference( PREF_KEY_MAX_CALLLIST_CALLS,  maxCalls );
            VM_SetPreference( PREF_KEY_SHOW_ACTIVATION, showActivation );

            if (this.m_callListView.UpdatePreferences( maxCalls, showActivation )) {
                this.updateView( );
            }
        }
        catch ( e )
        {
            EX_ASSERT_NO_EXCEPTIONS( e, 'VM_WidgetViewController::UpdatePreferences( )' );
        }
    },

    /**************************************************************************
      Event handler methods
     **************************************************************************/

    handleRegistrationSubmit:function( )
    {
        this.m_registerView.handleRegistrationSubmit( );
    },

    handleAuthenticateSubmit:function( )
    {
        this.m_loginView.handleAuthenticateSubmit( );
    },

    handlePinReminderRequested:function( )
    {
        this.m_loginView.handlePinReminderRequested( );
    },

    OnShow : function( )
    {
        try
        {
            if ( this.m_state == this.VIEW_STATE_SHOW_CALL_LIST )
            {
                this.m_callListView.StartSynchronization( );
            }
        }
        catch ( e )
        {
            EX_Log( "VM_WidgetViewController::OnShow( ) : " + e.message );
        }
    },

    OnHide : function( )
    {
        try
        {
            if ( this.m_state == this.VIEW_STATE_SHOW_CALL_LIST )
            {
                this.m_callListView.StartMinimalSynchronization( );
            }
        }
        catch ( e )
        {
            EX_Log( "VM_WidgetViewController::OnHide( ) : " + e.message );
        }
    },

    /**************************************************************************
      State management methods
     **************************************************************************/

    getState:function( )
    {
        return this.m_state;
    },

    getName: function()
    {
        return this.m_name;
    },

    getPhoneNumber: function()
    {
        return this.m_phoneNumber;
    },

    getEmailAddress: function()
    {
        return this.m_emailAddress;
    },

    setRegistrationSucceeded:function( )
    {
        this.m_loginView.handleGetAccountInfo( );
    },

    setLoginSucceeded:function(  )
    {
        this.m_loginView.handleGetAccountInfo( );
    },

    Logout : function( )
    {
        this.setLoginRequired( );
        this.resetAccountInfo( );
    },

    setLoginRequired:function( )
    {
        this.m_state = this.VIEW_STATE_SHOW_LOGIN;
        this.m_loginView.resetLoginForm( );
        this.updateView( );
    },
    
    setAutoLogin: function( )
    {
        this.m_state = this.VIEW_STATE_SHOW_LOGIN;
        this.m_loginView.handleGetAccountInfo( true );
        this.updateView( );
    },

    setRegistrationRequired:function( )
    {
        this.m_state = this.VIEW_STATE_SHOW_REGISTER;
        this.updateView( );
    },

    OnLoginRequired:function( statusMessage )
    {
        try
        {
            this.m_state = this.VIEW_STATE_SHOW_LOGIN;

            this.updateView( );

            this.m_loginView.SetStatusText( statusMessage );
        }
        catch ( e )
        {
            EX_Log( "VM_WidgetViewController::OnLoginRequired( ) : " + e.message );
        }
    },

    OnAuthenticationSucceeded:function( responseMessage )
    {
        try
        {
            // copy over all the attributes
            this.m_name                 = responseMessage.m_name;
            this.m_emailAddress         = responseMessage.m_email;
            this.m_phoneNumber          = responseMessage.m_phone;

            // set up the pref's details
            if (document.getElementById( 'WIDGET_personalInfo' ))
            {
                var userInformation = '';
                if (this.m_phoneNumber) {
                    userInformation +=
                      tag('tr','',
                        tag('td', {'class':'heading'}, "Cell Number") +
                        tag('td', {'class':'userInformation'}, this.m_phoneNumber.replace(/^(\d{3})(\d{3})(\d{4})$/, "$1-$2-$3" ) )); // mobile phone number (10 digits)
                }
                if (this.m_name) {
                    userInformation +=
                      tag('tr','',
                        tag('td', {'class':'heading'}, "Name") +
                        tag('td', {'class':'userInformation'}, this.m_name ));
                }
                if (this.m_emailAddress) {
                    userInformation +=
                      tag('tr','',
                        tag('td', {'class':'heading'}, "Email Address") +
                        tag('td', {'class':'userInformation'}, this.m_emailAddress ));
                }
                if (userInformation.length) {
                    document.getElementById( 'WIDGET_personalInfo' ).innerHTML =
                      tag('table', {width:'100%'}, userInformation );
                }
            }

            this.m_callListView.OnLoginSucceeded( responseMessage.m_secondsBetweenSync,
                                                  responseMessage.m_actStatus,
                                                  responseMessage.m_carrierId,
                                                  responseMessage.m_fwdCodes,
                                                  responseMessage.m_carriers,
                                                  responseMessage.m_hideRegCarrier );

            this.m_state = this.VIEW_STATE_SHOW_CALL_LIST;

            this.updateView( );
        }
        catch( e )
        {
            EX_Log( "VM_WidgetViewController::OnAuthenticationSucceeded( ) : " + e.message );
        }
    },

    /**************************************************************************
      View update methods
     **************************************************************************/

    resetViews:function( )
    {
        this.m_callListView.Hide( );
        this.m_registerView.reset( );
        this.m_loginView.reset( );
    },

    updateView:function( )
    {
        this.resetViews( );

        switch ( this.getState( ) )
        {
        case this.VIEW_STATE_SHOW_REGISTER:
            this.drawRegisterView( );
            break;

        case this.VIEW_STATE_SHOW_LOGIN:
            this.drawLoginView( );
            break;

        case this.VIEW_STATE_SHOW_CALL_LIST:
            this.drawCallListView( );
            break;

        default:
            this.drawRegisterView( );
            break;
        }
        if (window._IG_AdjustIFrameHeight) window._IG_AdjustIFrameHeight();
    },

    drawRegisterView:function( )
    {
        this.m_callListView.DisableSynchronization( );
        this.m_registerView.refresh( );
    },

    drawLoginView:function( )
    {
        this.m_callListView.DisableSynchronization( );
        this.m_loginView.refresh( );
    },

    drawCallListView:function( )
    {
        this.m_callListView.StartSynchronization( );
        this.m_callListView.Show( );
        this.m_callListView.refresh( );
    }
};

